#include "lees_grid.h"

void LeesGrid::ShowGrid()
{
	for (int y = 0; y < grid_size_y; y++)
	{
		for (int x = 0; x < grid_size_x; x++)
		{
			if (grid[y][x] > 9 || grid[y][x] < 0)
			{
				cout << grid[y][x] << " ";
			}
			else
			{
				cout << " " << grid[y][x] << " ";
			}
		}
		cout << endl;
		cout << endl;
	}
}

void LeesGrid::Clean()
{
	for (int y = 0; y < grid_size_y; y++)
	{
		for (int x = 0; x < grid_size_x; x++)
		{
			delete grid[y];
		}
	}
}

void LeesGrid::SetGrid()
{
	grid = new int*[grid_size_y];
	for (int i = 0; i < grid_size_y; i++)
	{
		grid[i] = new int[grid_size_x];
	}

	int random;

	srand(time(NULL));

	for (int y = 0; y < grid_size_y; y++)
	{
		for (int x = 0; x < grid_size_x; x++)
		{
			random = rand() % 100;
			if (random >= 80)
			{
				grid[y][x] = -2;
			}
			else
			{
				grid[y][x] = -1;
			}
		}
	}
}

void LeesGrid::CalculateRoute()
{
	srand(time(NULL));

	start_coord.x = rand() % grid_size_x;
	start_coord.y = rand() % grid_size_y;

	end_coord.x = rand() % grid_size_x;
	end_coord.y = rand() % grid_size_y;

	cout << "\nFinding route from (" << start_coord.x << ", " << start_coord.y << ") to (" << end_coord.x << ", " << end_coord.y << ")\n";

	if (PhaseOne())
	{
		PhaseTwo();
	}
}

void LeesGrid::CalculateRouteNoDisplay()
{
	srand(time(NULL));

	start_coord.x = rand() % grid_size_x;
	start_coord.y = rand() % grid_size_y;

	end_coord.x = rand() % grid_size_x;
	end_coord.y = rand() % grid_size_y;

	if (PhaseOneFast())
	{
		PhaseTwo();
	}
}

bool LeesGrid::PhaseOne()
{
	int numObstacles = 0;

	for (int y = 0; y < grid_size_y; y++)
	{
		for (int x = 0; x < grid_size_x; x++)
		{
			if (grid[y][x] == -2)
			{
				numObstacles++;
			}
		}
	}

	if (start_coord.x < 0 || start_coord.x >= grid_size_x)
	{
		cout << "The start coordinate is not on the grid." << endl;
		return false;
	}
	else if (start_coord.y < 0 || start_coord.y >= grid_size_y)
	{
		cout << "The start coordinate is not on the grid." << endl;
		return false;
	}
	else if (end_coord.x < 0 || start_coord.x >= grid_size_x)
	{
		cout << "The end coordinate is not on the grid." << endl;
		return false;
	}
	else if (end_coord.y < 0 || start_coord.y >= grid_size_y)
	{
		cout << "The end coordinate is not on the grid." << endl;
		return false;
	}

	grid[start_coord.y][start_coord.x] = 0;

	int route_length = 0;

	while (route_length < ((grid_size_x * grid_size_y) - numObstacles))
	{
		for (int y = 0; y < grid_size_y; y++)
		{
			for (int x = 0; x < grid_size_x; x++)
			{
				if (grid[y][x] == route_length)
				{
					if (y - 1 >= 0)
					{
						if (grid[y - 1][x])
						{
							grid[y - 1][x] = route_length + 1;
							if (y - 1 == end_coord.y && x == end_coord.x)
							{
								return true;
							}
						}
					}

					if (y + 1 < grid_size_y)
					{
						if (grid[y + 1][x] == -1)
						{
							grid[y + 1][x] = route_length + 1;
							if (y + 1 == end_coord.y && x == end_coord.x)
							{
								return true;
							}
						}
					}

					if (x - 1 >= 0)
					{
						if (grid[y][x - 1] == -1)
						{
							grid[y][x - 1] = route_length + 1;
							if (y == end_coord.y && x - 1 == end_coord.x)
							{
								return true;
							}
						}
					}

					if (x + 1 < grid_size_x)
					{
						if (grid[y][x + 1] == -1)
						{
							grid[y][x + 1] = route_length + 1;
							if (y == end_coord.y && x + 1 == end_coord.x)
							{
								return true;
							}
						}
					}
				}
			}
		}
		/*
		ShowGrid(grid);
		std::cin.ignore();
		*/

		route_length++;
	}

	cout << "There is no route from the start point to the end point." << endl << endl;
	return false;
}

void LeesGrid::PhaseTwo()
{
	std::vector<coord> route_list;

	int route_length = grid[end_coord.y][end_coord.x];

	int route_position = route_length;

	coord current_position = { end_coord.x, end_coord.y };

	route_list.push_back(current_position);

	while (route_position > 0)
	{
		if (current_position.y - 1 >= 0 && current_position.y + 1 < grid_size_y)
		{
			if (grid[current_position.y - 1][current_position.x] == route_position - 1)
			{
				route_list.push_back(current_position);
				current_position.x = current_position.x;
				current_position.y = current_position.y - 1;
				route_position--;
			}
			else if (grid[current_position.y + 1][current_position.x] == route_position - 1)
			{
				route_list.push_back(current_position);
				current_position.x = current_position.x;
				current_position.y = current_position.y + 1;
				route_position--;
			}
		}

		if (current_position.x - 1 >= 0 && current_position.x + 1 < grid_size_y)
		{
			if (grid[current_position.y][current_position.x - 1] == route_position - 1 && current_position.x - 1 >= 0)
			{
				route_list.push_back(current_position);
				current_position.x = current_position.x - 1;
				current_position.y = current_position.y;
				route_position--;
			}
			else if (grid[current_position.y][current_position.x + 1] == route_position - 1 && current_position.x + 1 < grid_size_x)
			{
				route_list.push_back(current_position);
				current_position.x = current_position.x + 1;
				current_position.y = current_position.y;
				route_position--;
			}
		}
	}

	route_list.push_back(start_coord);

	for (int i = 0; i < route_list.size(); i++)
	{
		grid[route_list[i].y][route_list[i].x] = -3;
	}

}

bool LeesGrid::PhaseOneFast()
{
	int numObstacles = 0;

	for (int y = 0; y < grid_size_y; y++)
	{
		for (int x = 0; x < grid_size_x; x++)
		{
			if (grid[y][x] == -2)
			{
				numObstacles++;
			}
		}
	}

	if (start_coord.x < 0 || start_coord.x >= grid_size_x)
	{
		return false;
	}
	else if (start_coord.y < 0 || start_coord.y >= grid_size_y)
	{
		return false;
	}
	else if (end_coord.x < 0 || start_coord.x >= grid_size_x)
	{
		return false;
	}
	else if (end_coord.y < 0 || start_coord.y >= grid_size_y)
	{
		return false;
	}

	grid[start_coord.y][start_coord.x] = 0;

	int route_length = 0;

	while (route_length < ((grid_size_x * grid_size_y) - numObstacles))
	{
		for (int y = 0; y < grid_size_y; y++)
		{
			for (int x = 0; x < grid_size_x; x++)
			{
				if (grid[y][x] == route_length)
				{
					if (y - 1 >= 0)
					{
						if (grid[y - 1][x])
						{
							grid[y - 1][x] = route_length + 1;
							if (y - 1 == end_coord.y && x == end_coord.x)
							{
								return true;
							}
						}
					}

					if (y + 1 < grid_size_y)
					{
						if (grid[y + 1][x] == -1)
						{
							grid[y + 1][x] = route_length + 1;
							if (y + 1 == end_coord.y && x == end_coord.x)
							{
								return true;
							}
						}
					}

					if (x - 1 >= 0)
					{
						if (grid[y][x - 1] == -1)
						{
							grid[y][x - 1] = route_length + 1;
							if (y == end_coord.y && x - 1 == end_coord.x)
							{
								return true;
							}
						}
					}

					if (x + 1 < grid_size_x)
					{
						if (grid[y][x + 1] == -1)
						{
							grid[y][x + 1] = route_length + 1;
							if (y == end_coord.y && x + 1 == end_coord.x)
							{
								return true;
							}
						}
					}
				}
			}
		}

		route_length++;
	}

	return false;
}
