#ifndef _VERTEX_H_
#define _VERTEX_H_

#include <string>
#include <list>
#include <queue>

class Vertex
{
public:
	struct Edge
	{
		Vertex* neighbour;
		int length;
		bool operator<(Edge& edge)
		{
			return ((this->neighbour->g_cost) < (edge.neighbour->g_cost));
		}
	};

public:
	Vertex(int index_, float posX_, float posY_, float posZ_) : index(index_), posX(posX_), posY(posY_), posZ(posZ_), blocked(false), distance(-1), h_cost(0), in_open(false){}

	int index;
	bool blocked;
	int distance;
	std::list<Edge *> neighbours;
	std::priority_queue<Edge *>ordered_neighbours;
	float posX, posY, posZ;
	float h_cost;
	float g_cost;
	bool in_open;

public:
	inline void connect(Vertex& target, int length)
	{
		Edge* edge = new Edge{ &target, length };

		neighbours.push_back(edge);
	}

	inline void CalculateHCost(Vertex& target)
	{
		h_cost = sqrt(pow((target.posX-posX), 2) + pow((target.posY-posY), 2) + pow((target.posZ-posZ), 2));
	}

	inline void CalculateGCost(int edge_length, int current_distance)
	{
		if (distance == -1 || current_distance + edge_length < distance)
		{
			g_cost = (current_distance + edge_length) + h_cost;
			distance = current_distance + edge_length;
		}
	}

	inline void OrderNeighbours()
	{
		for (auto v : neighbours)
		{
			v->neighbour->CalculateGCost(v->length, distance);
			ordered_neighbours.push(v);
		}
	}
};

#endif